/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Message.h>
#include <Core/MessageBroker/MessageBroker.h>
#include <Core/SkywardHubCore.h>

#include <QGridLayout>
#include <QLabel>
#include <QString>
#include <QTime>
#include <QTimer>
#include <QWidget>
#include <QtWidgets>

enum ErrorType
{
    ET_ERROR,
    ET_WARNING,
    ET_INFO,
};

class ErrorDisplayer;

class Error : public QWidget
{
    Q_OBJECT

public:
    Error(ErrorDisplayer* displayer, QString title, QString description,
          ErrorType type, int msecDuration);
    ~Error();
    void incrementCount(int msecDuration);
    bool compareError(QString title, QString description, ErrorType type);

protected:
    void mouseReleaseEvent(QMouseEvent*) override;
    void paintEvent(QPaintEvent* event) override;
    void decrementCount();

private:
    ErrorDisplayer* displayer;
    QLabel titlePrefix;
    QLabel title;
    QLabel description;
    QLabel countLabel;
    QGridLayout layout;
    ErrorType type;
    int count;
};
