/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Message.h>
#include <Core/XmlObject.h>

#include <QComboBox>
#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QMap>
#include <QString>

class MessageFormElement
{
public:
    MessageFormElement();

    ~MessageFormElement();

    /**
     * @brief Adds a new combo box field to the form.
     *
     * @return false if there is already a field with the given id.
     */
    bool addComboBox(QString id, QString label,
                     const QMap<QString, int>& options);

    /**
     * @brief Adds a new float field to the form.
     *
     * @return false if there is already a field with the given id.
     */
    bool addFloat(QString id, QString label,
                  float min    = std::numeric_limits<float>::min(),
                  float max    = std::numeric_limits<float>::max(),
                  int decimals = 2);

    /**
     * @brief Adds a new integer field to the form.
     *
     * @return false if there is already a field with the given id.
     */
    bool addInteger(QString id, QString label,
                    int min = std::numeric_limits<int>::min(),
                    int max = std::numeric_limits<int>::max());

    XmlObject toXmlObject(XmlObject& obj);

    void fromXmlObject(const XmlObject& obj);

    void applyToGridLayout(QGridLayout* layout);

    void removeFromGridLayout(QGridLayout* layout);

    Message prepareMessage(const QString& messageId);

private:
    QMap<QString, QPair<QLabel*, QComboBox*>> comboBoxMap;
    QMap<QString, std::tuple<QLabel*, QLineEdit*, float, float, int>> floatMap;
    QMap<QString, std::tuple<QLabel*, QLineEdit*, int, int>> integerMap;
};
