/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Window.h"

#include <Core/Module/Module.h>
#include <Core/ModulesManager/ModulesManager.h>

#include <QVBoxLayout>

#include "ui_Window.h"

Window::Window(QWidget* parent) : QWidget(parent), ui(new Ui::Window)
{
    ui->setupUi(this);
    errorDisplayer = new ErrorDisplayer(this);
}

Window::~Window()
{
    delete ui;
    delete errorDisplayer;
}

void Window::updateModule(Module* m)
{
    if (m != nullptr)
    {
        if (centralModule == nullptr || centralModule != m)
        {
            setCentralModule(m);
        }
    }
}

Module* Window::getCentralModule() const { return centralModule; }

void Window::setCentralModule(Module* value)
{
    if (centralModule != nullptr)
    {
        // ui->mainLayout->removeWidget(centralModule->toWidget());
        disconnect(centralModule->getEventHandler(), &QWidget::destroyed, this,
                   &Window::onWidgetDestroyed);
        disconnect(centralModule->getEventHandler(),
                   &EventHandler::replaceMeWith, this,
                   &Window::onReplaceRequested);
    }
    centralModule = value;
    ui->mainLayout->addWidget(value->toWidget());
    connect(value->getEventHandler(), &QWidget::destroyed, this,
            &Window::onWidgetDestroyed);
    connect(value->getEventHandler(), &EventHandler::replaceMeWith, this,
            &Window::onReplaceRequested);

    value->toWidget()->lower();
}

void Window::closeEvent(QCloseEvent* event)
{
    if (centralModule != nullptr)
        delete centralModule;

    QWidget::closeEvent(event);

    deleteLater();
}

void Window::onWidgetDestroyed()
{
    centralModule = nullptr;
    close();
    this->deleteLater();
}

void Window::onReplaceRequested(Module* sender, Module* newModule)
{
    if (newModule != nullptr && sender != nullptr)
    {
        setCentralModule(newModule);
        delete sender;
    }
}

void Window::resizeEvent(QResizeEvent* event)
{
    Q_UNUSED(event);
    errorDisplayer->reposition();
}
