/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Modules/DefaultModule/DefaultModule.h>
#include <Qt3DRender/qpointlight.h>

#include <QSizePolicy>
#include <Qt3DCore/QAspectEngine>
#include <Qt3DCore/QEntity>
#include <Qt3DCore/QTransform>
#include <Qt3DExtras/QConeMesh>
#include <Qt3DExtras/QCuboidMesh>
#include <Qt3DExtras/QFirstPersonCameraController>
#include <Qt3DExtras/QForwardRenderer>
#include <Qt3DExtras/QOrbitCameraController>
#include <Qt3DExtras/QPhongMaterial>
#include <Qt3DExtras/QSphereMesh>
#include <Qt3DExtras/QTorusMesh>
#include <Qt3DExtras/Qt3DWindow>
#include <Qt3DInput/QInputAspect>
#include <Qt3DRender/QAttribute>
#include <Qt3DRender/QBuffer>
#include <Qt3DRender/QCamera>
#include <Qt3DRender/QCameraLens>
#include <Qt3DRender/QGeometry>
#include <Qt3DRender/QGeometryRenderer>
#include <Qt3DRender/QRenderAspect>

class OrientationVisualizer : public DefaultModule
{
    Q_OBJECT

public:
    explicit OrientationVisualizer(QWidget *parent = nullptr);

    QWidget *toWidget() override;

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

private:
    QLabel *rollLabel, *pitchLabel, *yawLabel;

    Qt3DCore::QTransform *rocketTransform;

    void updateOrientation(double qx, double qy, double qz, double qw);
    void setupUi();

    void drawLine(const QVector3D &start, const QVector3D &end,
                  const QColor &color, Qt3DCore::QEntity *rootEntity);

    QQuaternion sceneRotation = {-0.5, -0.5, -0.5, 0.5};
};
