/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QListWidget>
#include <QStackedWidget>
#include <QWidget>

#include "Modules/DefaultModule/DefaultModule.h"

class TabsModule : public DefaultModule
{
    Q_OBJECT
public:
    TabsModule(QWidget* parent = nullptr);
    ~TabsModule();

    void addTab(const QString& tabName, Module* module);
    QString generateUniqueTabName();

    void addCustomActionsToMenu() override;

    QWidget* toWidget() override;
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject& xmlObject) override;

protected slots:
    void onTabDeleted(Module* module);
    void replaceTabContent(Module* from, Module* to);
    void onDoubleClickedItem(QListWidgetItem* item);

    void onMenuNewTabClick();
    void onMenuDeleteTabClick();
    void onMenuRenameTabClick();

private:
    QListWidget* tabNames;
    QStackedWidget* tabContents;
    QList<Module*> contentModules;

    /* number of tabs ever created (used for names) */
    unsigned totalTabs = 0;

    void setupUi();
};
