/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QLabel>
#include <QWidget>

#include "Core/Message/Message.h"
#include "Modules/DefaultModule/DefaultModule.h"
#include "ValueElement.h"

class ValuesViewerConfigPanel;

namespace Ui
{
class ValuesConverterViewerModule;
}

class ValuesConverterViewerModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit ValuesConverterViewerModule(QWidget* parent = nullptr);
    ~ValuesConverterViewerModule();

    QWidget* toWidget() override;

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject& xmlObject) override;

protected:
    void onConfigureClicked();
    void onConfigurationSaved(ValuesViewerConfigPanel* sPanel);
    void addCustomActionsToMenu() override;
    QLabel* createView(const ValueElement& el) const;
    void clearLabels();
    void createLabels();
    void addRule(const ValueElement& el);
    void setRules(const QList<ValueElement>& rList);
    void onMsgReceived(const Message& msg);
    void clearRules();
    int findLabelIndexByName(const QString& name);
    bool isTopicPresent(const QString& topic);

private:
    Ui::ValuesConverterViewerModule* ui;
    int minColNumber = 1;
    int columns      = minColNumber;
    QList<QLabel*> labels;
    QList<ValueElement> rules;
};
